@@ -1,4 +1,4 @@
-/* Copyright (C) 2010 by Cristian Henzel <oss@web-tm.com>
+/* Copyright (C) 2010 by Cristian Henzel <oss@rspwn.com>
  *
  * forked from parcellite, which is
  * Copyright (C) 2007-2008 by Xyhthyx <xyhthyx@gmail.com>
@@ -25,7 +25,6 @@
 
 #include <glib.h>
 #include <stdlib.h>
-#include <unistd.h>
 #include <gtk/gtk.h>
 #ifdef HAVE_APPINDICATOR
 #include <libappindicator/app-indicator.h>
@@ -44,18 +43,25 @@ static gchar* clipboard_text;
 static gchar* synchronized_text;
 static GtkClipboard* primary;
 static GtkClipboard* clipboard;
-static GtkStatusIcon* status_icon;
+#ifdef HAVE_APPINDICATOR
+static AppIndicator *indicator;
+static GtkWidget *indicator_menu = NULL;
+#else
+static GtkStatusIcon *status_icon; 
+static GtkWidget *statusicon_menu = NULL;
+static gboolean status_menu_lock = FALSE;
+#endif
 
 static gboolean actions_lock = FALSE;
 
 /* Init preferences structure */
 prefs_t prefs = {DEF_USE_COPY,        DEF_USE_PRIMARY,      DEF_SYNCHRONIZE,
                  DEF_SHOW_INDEXES,    DEF_SAVE_URIS,        DEF_SAVE_HISTORY,
-                 DEF_HISTORY_LIMIT,   DEF_SMALL_HISTORY,    DEF_HISTORY_SMALL,
-                 DEF_HYPERLINKS_ONLY, DEF_CONFIRM_CLEAR,    DEF_FULL_HIST_BUTTON,
-                 DEF_SINGLE_LINE,     DEF_REVERSE_HISTORY,  DEF_ITEM_LENGTH,
-                 DEF_ELLIPSIZE,       INIT_HISTORY_KEY,     INIT_ACTIONS_KEY,
-                 INIT_MENU_KEY,       INIT_SEARCH_KEY,      DEF_NO_ICON};
+                 DEF_HISTORY_LIMIT,   DEF_ITEMS_MENU,       DEF_HYPERLINKS_ONLY,
+                 DEF_CONFIRM_CLEAR,   DEF_SINGLE_LINE,      DEF_REVERSE_HISTORY,
+                 DEF_ITEM_LENGTH,     DEF_ELLIPSIZE,        INIT_HISTORY_KEY,
+                 INIT_ACTIONS_KEY,    INIT_MENU_KEY,        INIT_SEARCH_KEY,
+                 DEF_NO_ICON};
 
 /* Called every CHECK_INTERVAL seconds to check for new items */
 static gboolean item_check(gpointer data)
@@ -129,7 +135,7 @@ static gboolean item_check(gpointer data)
     /* Check contents */
     gint count;
     GdkAtom *targets;
-    gboolean contents = gtk_clipboard_wait_for_targets(primary, &targets, &count);
+    gboolean contents = gtk_clipboard_wait_for_targets(clipboard, &targets, &count);
     g_free(targets);
     /* Only recover lost contents if there isn't any other type of content in the clipboard */
     if (!contents)
@@ -184,7 +190,7 @@ static gboolean item_check(gpointer data)
   /* Cleanup */
   g_free(primary_temp);
   g_free(clipboard_temp);
-  
+
   return TRUE;
 }
 
@@ -194,17 +200,7 @@ static void *execute_action(void *command)
   /* Execute action */
   int sys_return;
   actions_lock = TRUE;
-  if (!prefs.no_icon)
-  {
-  gtk_status_icon_set_from_stock((GtkStatusIcon*)status_icon, GTK_STOCK_EXECUTE);
-  gtk_status_icon_set_tooltip((GtkStatusIcon*)status_icon, _("Executing action..."));
-  }
   sys_return = system((gchar*)command);
-  if (!prefs.no_icon)
-  {
-  gtk_status_icon_set_from_stock((GtkStatusIcon*)status_icon, GTK_STOCK_PASTE);
-  gtk_status_icon_set_tooltip((GtkStatusIcon*)status_icon, _("Clipboard Manager"));
-  }
   actions_lock = FALSE;
   g_free((gchar*)command);
   /* Exit this thread */
@@ -215,25 +211,15 @@ static void *execute_action(void *command)
 static void action_exit(GPid pid, gint status, gpointer data)
 {
   g_spawn_close_pid(pid);
-  if (!prefs.no_icon)
-  {
-    gtk_status_icon_set_from_stock((GtkStatusIcon*)status_icon, GTK_STOCK_PASTE);
-    gtk_status_icon_set_tooltip((GtkStatusIcon*)status_icon, _("Clipboard Manager"));
-  }
   actions_lock = FALSE;
 }
 
 /* Called when an action is selected from actions menu */
 static void action_selected(GtkButton *button, gpointer user_data)
 {
-  /* Change icon and enable lock */
+  /* Enable lock */
   actions_lock = TRUE;
-  if (!prefs.no_icon)
-  {
-    gtk_status_icon_set_from_stock((GtkStatusIcon*)status_icon, GTK_STOCK_EXECUTE);
-    gtk_status_icon_set_tooltip((GtkStatusIcon*)status_icon, _("Executing action..."));
-  }
-  
+
   /* Insert clipboard into command (user_data), and prepare it for execution */
   gchar* clipboard_text = gtk_clipboard_wait_for_text(clipboard);
   gchar* command = g_markup_printf_escaped((gchar*)user_data, clipboard_text);
@@ -267,56 +253,21 @@ static void edit_actions_selected(GtkButton *button, gpointer user_data)
 static void item_selected(GtkMenuItem *menu_item, gpointer user_data)
 {
   /* Get the text from the right element and set as clipboard */
-  GSList* element = g_slist_nth(history, (gint64)user_data);
+  GSList* element = g_slist_nth(history, GPOINTER_TO_INT(user_data));
   gtk_clipboard_set_text(clipboard, (gchar*)element->data, -1);
   gtk_clipboard_set_text(primary, (gchar*)element->data, -1);
 }
 
-/* Called when Clear is selected from history menu */
-static void clear_selected(GtkMenuItem *menu_item, gpointer user_data)
-{
-  /* Check for confirm clear option */
-  if (prefs.confirm_clear)
-  {
-    GtkWidget* confirm_dialog = gtk_message_dialog_new(NULL,
-                                                       GTK_DIALOG_MODAL,
-                                                       GTK_MESSAGE_OTHER,
-                                                       GTK_BUTTONS_OK_CANCEL,
-                                                       _("Clear the history?"));
-    gtk_window_set_title((GtkWindow*)confirm_dialog, _("Clear history"));
-    
-    if (gtk_dialog_run((GtkDialog*)confirm_dialog) == GTK_RESPONSE_OK)
-    {
-      /* Clear history and free history-related variables */
-      g_slist_free(history);
-      history = NULL;
-      save_history();
-      g_free(primary_text);
-      g_free(clipboard_text);
-      g_free(synchronized_text);
-      primary_text = g_strdup("");
-      clipboard_text = g_strdup("");
-      synchronized_text = g_strdup("");
-      gtk_clipboard_set_text(primary, "", -1);
-      gtk_clipboard_set_text(clipboard, "", -1);
-    }
-    gtk_widget_destroy(confirm_dialog);
-  }
-  else
-  {
-    /* Clear history and free history-related variables */
-    g_slist_free(history);
-    history = NULL;
-    save_history();
-    g_free(primary_text);
-    g_free(clipboard_text);
-    g_free(synchronized_text);
-    primary_text = g_strdup("");
-    clipboard_text = g_strdup("");
-    synchronized_text = g_strdup("");
-    gtk_clipboard_set_text(primary, "", -1);
-    gtk_clipboard_set_text(clipboard, "", -1);
-  }
+/* Clears all local data (specific to main.c) */
+void clear_main_data() {
+  g_free(primary_text);
+  g_free(clipboard_text);
+  g_free(synchronized_text);
+  primary_text = g_strdup("");
+  clipboard_text = g_strdup("");
+  synchronized_text = g_strdup("");
+  gtk_clipboard_set_text(primary, "", -1);
+  gtk_clipboard_set_text(clipboard, "", -1);
 }
 
 /* Called when About is selected from right-click menu */
@@ -325,7 +276,7 @@ static void show_about_dialog(GtkMenuItem *menu_item, gpointer user_data)
   /* This helps prevent multiple instances */
   if (!gtk_grab_get_current())
   {
-    const gchar* authors[] = {"Cristian Henzel <oss@web-tm.com>\n"
+    const gchar* authors[] = {"Cristian Henzel <oss@rspwn.com>\n"
 				"Gilberto \"Xyhthyx\" Miralla <xyhthyx@gmail.com>", NULL};
     const gchar* license =
       "This program is free software; you can redistribute it and/or modify\n"
@@ -354,7 +305,7 @@ static void show_about_dialog(GtkMenuItem *menu_item, gpointer user_data)
                                 _("Lightweight GTK+ clipboard manager."));
     
     gtk_about_dialog_set_website((GtkAboutDialog*)about_dialog,
-                                 "http://gtkclipit.sourceforge.net/");
+                                 "http://clipit.rspwn.com/");
     
     gtk_about_dialog_set_copyright((GtkAboutDialog*)about_dialog, "Copyright (C) 2010 Cristian Henzel");
     gtk_about_dialog_set_authors((GtkAboutDialog*)about_dialog, authors);
@@ -514,108 +465,11 @@ static gboolean show_actions_menu(gpointer data)
   return FALSE;
 }
 
-static gboolean show_history_menu_full(gpointer data)
-{
-  /* Declare some variables */
-  GtkWidget *menu,       *menu_item,
-            *menu_image, *item_label;
-  
-  /* Create the menu */
-  menu = gtk_menu_new();
-  g_signal_connect((GObject*)menu, "selection-done", (GCallback)gtk_widget_destroy, NULL);
-  /* Items */
-  if ((history != NULL) && (history->data != NULL))
-  {
-    /* Declare some variables */
-    GSList* element;
-    gint64 element_number = 0;
-    gchar* primary_temp = gtk_clipboard_wait_for_text(primary);
-    gchar* clipboard_temp = gtk_clipboard_wait_for_text(clipboard);
-    /* Reverse history if enabled */
-    if (prefs.reverse_history)
-    {
-      history = g_slist_reverse(history);
-      element_number = g_slist_length(history) - 1;
-    }
-    /* Go through each element and adding each */
-    for (element = history; element != NULL; element = element->next)
-    {
-      GString* string = g_string_new((gchar*)element->data);
-      /* Ellipsize text */
-      string = ellipsize_string(string);
-      /* Remove control characters */
-      string = remove_newlines_string(string);
-      /* Make new item with ellipsized text */
-      gchar* list_item;
-      if (prefs.show_indexes)
-      {
-        list_item = g_strdup_printf("%" G_GINT64_FORMAT ". %s", (element_number+1), string->str);
-      } else {
-        list_item = g_strdup(string->str);
-      }
-      menu_item = gtk_menu_item_new_with_label(list_item);
-      g_signal_connect((GObject*)menu_item,      "activate",
-                       (GCallback)item_selected, (gpointer)element_number);
-      /* Modify menu item label properties */
-      item_label = gtk_bin_get_child((GtkBin*)menu_item);
-      gtk_label_set_single_line_mode((GtkLabel*)item_label, prefs.single_line);
-      
-      /* Check if item is also clipboard text and make bold */
-      if ((clipboard_temp) && (g_strcmp0((gchar*)element->data, clipboard_temp) == 0))
-      {
-        gchar* bold_text = g_markup_printf_escaped("<b>%s</b>", list_item);
-        gtk_label_set_markup((GtkLabel*)item_label, bold_text);
-        g_free(bold_text);
-      }
-      else if ((primary_temp) && (g_strcmp0((gchar*)element->data, primary_temp) == 0))
-      {
-        gchar* italic_text = g_markup_printf_escaped("<i>%s</i>", list_item);
-        gtk_label_set_markup((GtkLabel*)item_label, italic_text);
-        g_free(italic_text);
-      }
-      g_free(list_item);
-      /* Append item */
-      gtk_menu_shell_append((GtkMenuShell*)menu, menu_item);
-      /* Prepare for next item */
-      g_string_free(string, TRUE);
-      if (prefs.reverse_history)
-        element_number--;
-      else
-        element_number++;
-    }
-    /* Cleanup */
-    g_free(primary_temp);
-    g_free(clipboard_temp);
-    /* Return history to normal if reversed */
-    if (prefs.reverse_history)
-      history = g_slist_reverse(history);
-  }
-  else
-  {
-    /* Nothing in history so adding empty */
-    menu_item = gtk_menu_item_new_with_label(_("Empty"));
-    gtk_widget_set_sensitive(menu_item, FALSE);
-    gtk_menu_shell_append((GtkMenuShell*)menu, menu_item);
-  }
-  /* -------------------- */
-  gtk_menu_shell_append((GtkMenuShell*)menu, gtk_separator_menu_item_new());
-  /* Clear */
-  menu_item = gtk_image_menu_item_new_from_stock(GTK_STOCK_CLEAR, NULL);
-  g_signal_connect((GObject*)menu_item, "activate", (GCallback)clear_selected, NULL);
-  gtk_menu_shell_append((GtkMenuShell*)menu, menu_item);
-  /* Popup the menu... */
-  gtk_widget_show_all(menu);
-  gtk_menu_popup((GtkMenu*)menu, NULL, NULL, NULL, NULL, 1, gtk_get_current_event_time());
-  /* Return FALSE so the g_timeout_add() function is called only once */
-  return FALSE;
-}
-
 /* Generates the small history menu */
-static gboolean show_history_menu_small(gpointer data)
+static gboolean show_history_menu(gpointer data)
 {
   /* Declare some variables */
-  GtkWidget *menu,       *menu_item,
-            *menu_image, *item_label;
+  GtkWidget *menu, *menu_item, *menu_image, *item_label;
   
   /* Create the menu */
   menu = gtk_menu_new();
@@ -625,8 +479,8 @@ static gboolean show_history_menu_small(gpointer data)
   {
     /* Declare some variables */
     GSList* element;
-    gint64 element_number = 0;
-    gint64 element_number_small = 0;
+    gint element_number = 0;
+    gint element_number_small = 0;
     gchar* primary_temp = gtk_clipboard_wait_for_text(primary);
     gchar* clipboard_temp = gtk_clipboard_wait_for_text(clipboard);
     /* Reverse history if enabled */
@@ -636,7 +490,7 @@ static gboolean show_history_menu_small(gpointer data)
       element_number = g_slist_length(history) - 1;
     }
     /* Go through each element and adding each */
-    for (element = history; (element != NULL) && (element_number_small < prefs.history_small); element = element->next)
+    for (element = history; (element != NULL) && (element_number_small < prefs.items_menu); element = element->next)
     {
       GString* string = g_string_new((gchar*)element->data);
       /* Ellipsize text */
@@ -647,13 +501,12 @@ static gboolean show_history_menu_small(gpointer data)
       gchar* list_item;
       if (prefs.show_indexes)
       {
-        list_item = g_strdup_printf("%" G_GINT64_FORMAT ". %s", (element_number_small+1), string->str);
+        list_item = g_strdup_printf("%d. %s", (element_number_small+1), string->str);
       } else {
         list_item = g_strdup(string->str);
       }
       menu_item = gtk_menu_item_new_with_label(list_item);
-      g_signal_connect((GObject*)menu_item,      "activate",
-                       (GCallback)item_selected, (gpointer)element_number);
+      g_signal_connect((GObject*)menu_item, "activate", (GCallback)item_selected, GINT_TO_POINTER(element_number));
       
       /* Modify menu item label properties */
       item_label = gtk_bin_get_child((GtkBin*)menu_item);
@@ -697,96 +550,30 @@ static gboolean show_history_menu_small(gpointer data)
     gtk_widget_set_sensitive(menu_item, FALSE);
     gtk_menu_shell_append((GtkMenuShell*)menu, menu_item);
   }
-  /* -------------------- */
-  /* Show full history button (if enabled) */
-  if (!prefs.full_hist_button)
-  {
-    gtk_menu_shell_append((GtkMenuShell*)menu, gtk_separator_menu_item_new());
-    menu_item = gtk_image_menu_item_new_with_mnemonic(_("_Show full history"));
-    menu_image = gtk_image_new_from_stock(GTK_STOCK_ZOOM_IN, GTK_ICON_SIZE_MENU);
-    gtk_image_menu_item_set_image((GtkImageMenuItem*)menu_item, menu_image);
-    g_signal_connect((GObject*)menu_item, "activate", (GCallback)show_history_menu_full, NULL);
-    gtk_menu_shell_append((GtkMenuShell*)menu, menu_item);
-  }
   /* Popup the menu... */
   gtk_widget_show_all(menu);
   gtk_menu_popup((GtkMenu*)menu, NULL, NULL, NULL, NULL, 1, gtk_get_current_event_time());
+  gtk_menu_shell_select_first((GtkMenuShell*)menu, TRUE);
   /* Return FALSE so the g_timeout_add() function is called only once */
   return FALSE;
 }
 
-/* Called when status icon is left-clicked */
-static gboolean show_history_menu()
-{
-  if (prefs.small_history)
-    g_timeout_add(POPUP_DELAY, show_history_menu_small, NULL);
-  else
-    g_timeout_add(POPUP_DELAY, show_history_menu_full, NULL);
-  /* Return FALSE so the g_timeout_add() function is called only once */
-  return FALSE;
-}
-
-/* Called when status icon is right-clicked */
-static void show_clipit_menu(GtkStatusIcon *status_icon, guint button, guint activate_time)
-{
-  /* Declare some variables */
-  GtkWidget *menu,       *menu_item,
-            *menu_image, *item_label;
-  
-  /* Create the menu */
-  menu = gtk_menu_new();
-  g_signal_connect((GObject*)menu, "selection-done", (GCallback)gtk_widget_destroy, NULL);
-  /* About */
-  menu_item = gtk_image_menu_item_new_from_stock(GTK_STOCK_ABOUT, NULL);
-  g_signal_connect((GObject*)menu_item, "activate", (GCallback)show_about_dialog, NULL);
-  gtk_menu_shell_append((GtkMenuShell*)menu, menu_item);
-  /* Full history button (if enabled) */
-  if (prefs.full_hist_button && prefs.small_history)
-  {
-    menu_item = gtk_image_menu_item_new_with_mnemonic(_("_Show full history"));
-    menu_image = gtk_image_new_from_stock(GTK_STOCK_ZOOM_IN, GTK_ICON_SIZE_MENU);
-    gtk_image_menu_item_set_image((GtkImageMenuItem*)menu_item, menu_image);
-    g_signal_connect((GObject*)menu_item, "activate", (GCallback)show_history_menu_full, NULL);
-    gtk_menu_shell_append((GtkMenuShell*)menu, menu_item);
-  }
-  /* Manage history */
-  menu_item = gtk_image_menu_item_new_with_mnemonic(_("_Manage history"));
-  menu_image = gtk_image_new_from_stock(GTK_STOCK_FIND, GTK_ICON_SIZE_MENU);
-  gtk_image_menu_item_set_image((GtkImageMenuItem*)menu_item, menu_image);
-  g_signal_connect((GObject*)menu_item, "activate", (GCallback)show_search, NULL);
-  gtk_menu_shell_append((GtkMenuShell*)menu, menu_item);
-  /* Preferences */
-  menu_item = gtk_image_menu_item_new_from_stock(GTK_STOCK_PREFERENCES, NULL);
-  g_signal_connect((GObject*)menu_item, "activate", (GCallback)preferences_selected, NULL);
-  gtk_menu_shell_append((GtkMenuShell*)menu, menu_item);
-  /* -------------------- */
-  gtk_menu_shell_append((GtkMenuShell*)menu, gtk_separator_menu_item_new());
-  /* Quit */
-  menu_item = gtk_image_menu_item_new_from_stock(GTK_STOCK_QUIT, NULL);
-  g_signal_connect((GObject*)menu_item, "activate", (GCallback)quit_selected, NULL);
-  gtk_menu_shell_append((GtkMenuShell*)menu, menu_item);
-  /* Popup the menu... */
-  gtk_widget_show_all(menu);
-  gtk_menu_popup((GtkMenu*)menu, NULL, NULL, NULL, NULL, button, activate_time);
-}
-
 #ifdef HAVE_APPINDICATOR
-static void create_app_indicator()
+
+void create_app_indicator(gint create)
 {
-	GtkWidget *menu,	*menu_item,
-		*menu_image,	*item_label;
-	AppIndicator *indicator;
+	GtkWidget *menu_item, *menu_image, *item_label;
 
 	/* Create the menu */
-	menu = gtk_menu_new();
-	g_signal_connect((GObject*)menu, "selection-done", (GCallback)gtk_widget_destroy, NULL);
+	indicator_menu = gtk_menu_new();
+	g_signal_connect((GObject*)indicator_menu, "selection-done", (GCallback)gtk_widget_destroy, NULL);
 	/* Items */
 	if ((history != NULL) && (history->data != NULL))
 	{
 		/* Declare some variables */
 		GSList* element;
-		gint64 element_number = 0;
-		gint64 element_number_small = 0;
+		gint element_number = 0;
+		gint element_number_small = 0;
 		gchar* primary_temp = gtk_clipboard_wait_for_text(primary);
 		gchar* clipboard_temp = gtk_clipboard_wait_for_text(clipboard);
 		/* Reverse history if enabled */
@@ -796,7 +583,7 @@ static void create_app_indicator()
 			element_number = g_slist_length(history) - 1;
 		}
 		/* Go through each element and adding each */
-		for (element = history; (element != NULL) && (element_number_small < prefs.history_small); element = element->next)
+		for (element = history; (element != NULL) && (element_number_small < prefs.items_menu); element = element->next)
 		{
 			GString* string = g_string_new((gchar*)element->data);
 			/* Ellipsize text */
@@ -807,14 +594,14 @@ static void create_app_indicator()
 			gchar* list_item;
 			if (prefs.show_indexes)
 			{
-				list_item = g_strdup_printf("%" G_GINT64_FORMAT ". %s",
+				list_item = g_strdup_printf("%d. %s",
 								(element_number_small+1), string->str);
 			} else {
 				list_item = g_strdup(string->str);
 			}
 			menu_item = gtk_menu_item_new_with_label(list_item);
 			g_signal_connect((GObject*)menu_item,		"activate",
-					(GCallback)item_selected,	(gpointer)element_number);
+					(GCallback)item_selected,	GINT_TO_POINTER(element_number));
 
 			/* Modify menu item label properties */
 			item_label = gtk_bin_get_child((GtkBin*)menu_item);
@@ -835,7 +622,7 @@ static void create_app_indicator()
 			}
 			g_free(list_item);
 			/* Append item */
-			gtk_menu_shell_append((GtkMenuShell*)menu, menu_item);
+			gtk_menu_shell_append((GtkMenuShell*)indicator_menu, menu_item);
 			/* Prepare for next item */
 			g_string_free(string, TRUE);
 			if (prefs.reverse_history)
@@ -856,47 +643,167 @@ static void create_app_indicator()
 		/* Nothing in history so adding empty */
 		menu_item = gtk_menu_item_new_with_label(_("Empty"));
 		gtk_widget_set_sensitive(menu_item, FALSE);
-		gtk_menu_shell_append((GtkMenuShell*)menu, menu_item);
+		gtk_menu_shell_append((GtkMenuShell*)indicator_menu, menu_item);
 	}
 	/* -------------------- */
-	gtk_menu_shell_append((GtkMenuShell*)menu, gtk_separator_menu_item_new());
+	gtk_menu_shell_append((GtkMenuShell*)indicator_menu, gtk_separator_menu_item_new());
 
 	menu_item = gtk_image_menu_item_new_from_stock(GTK_STOCK_ABOUT, NULL);
 	g_signal_connect((GObject*)menu_item, "activate", (GCallback)show_about_dialog, NULL);
-	gtk_menu_shell_append((GtkMenuShell*)menu, menu_item);
-	/* Full history button (if enabled) */
-	menu_item = gtk_image_menu_item_new_with_mnemonic(_("_Show full history"));
-	menu_image = gtk_image_new_from_stock(GTK_STOCK_ZOOM_IN, GTK_ICON_SIZE_MENU);
-	gtk_image_menu_item_set_image((GtkImageMenuItem*)menu_item, menu_image);
-	g_signal_connect((GObject*)menu_item, "activate", (GCallback)show_history_menu_full, NULL);
-	gtk_menu_shell_append((GtkMenuShell*)menu, menu_item);
+	gtk_menu_shell_append((GtkMenuShell*)indicator_menu, menu_item);
 	/* Manage history */
 	menu_item = gtk_image_menu_item_new_with_mnemonic(_("_Manage history"));
 	menu_image = gtk_image_new_from_stock(GTK_STOCK_FIND, GTK_ICON_SIZE_MENU);
 	gtk_image_menu_item_set_image((GtkImageMenuItem*)menu_item, menu_image);
 	g_signal_connect((GObject*)menu_item, "activate", (GCallback)show_search, NULL);
-	gtk_menu_shell_append((GtkMenuShell*)menu, menu_item);
+	gtk_menu_shell_append((GtkMenuShell*)indicator_menu, menu_item);
 	/* Preferences */
 	menu_item = gtk_image_menu_item_new_from_stock(GTK_STOCK_PREFERENCES, NULL);
 	g_signal_connect((GObject*)menu_item, "activate", (GCallback)preferences_selected, NULL);
-	gtk_menu_shell_append((GtkMenuShell*)menu, menu_item);
+	gtk_menu_shell_append((GtkMenuShell*)indicator_menu, menu_item);
 	/* Quit */
 	menu_item = gtk_image_menu_item_new_from_stock(GTK_STOCK_QUIT, NULL);
 	g_signal_connect((GObject*)menu_item, "activate", (GCallback)quit_selected, NULL);
-	gtk_menu_shell_append((GtkMenuShell*)menu, menu_item);
+	gtk_menu_shell_append((GtkMenuShell*)indicator_menu, menu_item);
 	/* show the menu... */
-	gtk_widget_show_all(menu);
-	indicator = app_indicator_new ("clipit",
-					"stock_paste",
-					APP_INDICATOR_CATEGORY_APPLICATION_STATUS);
-
-	app_indicator_set_status (indicator, APP_INDICATOR_STATUS_ACTIVE);
-	app_indicator_set_attention_icon (indicator, "stock_paste");
-	app_indicator_set_menu (indicator, GTK_MENU (menu));
+	gtk_widget_show_all(indicator_menu);
+	/* check if we need to create the indicator or just refresh the menu */
+	if(create == 1) {
+		indicator = app_indicator_new ("clipit", "stock_paste", APP_INDICATOR_CATEGORY_APPLICATION_STATUS);
+		app_indicator_set_status (indicator, APP_INDICATOR_STATUS_ACTIVE);
+		app_indicator_set_attention_icon (indicator, "stock_paste");
+	}
+	app_indicator_set_menu (indicator, GTK_MENU (indicator_menu));
 }
+
 #else
+
 /* Called when status icon is clicked */
-/* (checks type of click and calls correct function */
+static gboolean show_clipit_menu()
+{
+	/* If the menu is visible, we don't do anything, so that it gets hidden */
+	if((statusicon_menu != NULL) && gtk_widget_get_visible((GtkWidget *)statusicon_menu))
+		return;
+	if(status_menu_lock)
+		return;
+	status_menu_lock = TRUE;
+
+	/* Declare some variables */
+	GtkWidget *menu_item, *menu_image, *item_label;
+
+	/* Create the menu */
+	statusicon_menu = gtk_menu_new();
+	g_signal_connect((GObject*)statusicon_menu, "selection-done", (GCallback)gtk_widget_destroy, NULL);
+
+	/* Items */
+	if ((history != NULL) && (history->data != NULL))
+	{
+		/* Declare some variables */
+		GSList* element;
+		gint element_number = 0;
+		gint element_number_small = 0;
+		gchar* primary_temp = gtk_clipboard_wait_for_text(primary);
+		gchar* clipboard_temp = gtk_clipboard_wait_for_text(clipboard);
+		/* Reverse history if enabled */
+		if (prefs.reverse_history)
+		{
+			history = g_slist_reverse(history);
+			element_number = g_slist_length(history) - 1;
+		}
+		/* Go through each element and adding each */
+		for (element = history; (element != NULL) && (element_number_small < prefs.items_menu); element = element->next)
+		{
+			GString* string = g_string_new((gchar*)element->data);
+			/* Ellipsize text */
+			string = ellipsize_string(string);
+			/* Remove control characters */
+			string = remove_newlines_string(string);
+			/* Make new item with ellipsized text */
+			gchar* list_item;
+			if (prefs.show_indexes)
+			{
+				list_item = g_strdup_printf("%d. %s",
+								(element_number_small+1), string->str);
+			} else {
+				list_item = g_strdup(string->str);
+			}
+			menu_item = gtk_menu_item_new_with_label(list_item);
+			g_signal_connect((GObject*)menu_item,		"activate",
+					(GCallback)item_selected,	GINT_TO_POINTER(element_number));
+
+			/* Modify menu item label properties */
+			item_label = gtk_bin_get_child((GtkBin*)menu_item);
+			gtk_label_set_single_line_mode((GtkLabel*)item_label, prefs.single_line);
+
+			/* Check if item is also clipboard text and make bold */
+			if ((clipboard_temp) && (g_strcmp0((gchar*)element->data, clipboard_temp) == 0))
+			{
+				gchar* bold_text = g_markup_printf_escaped("<b>%s</b>", list_item);
+				gtk_label_set_markup((GtkLabel*)item_label, bold_text);
+				g_free(bold_text);
+			}
+			else if ((primary_temp) && (g_strcmp0((gchar*)element->data, primary_temp) == 0))
+			{
+				gchar* italic_text = g_markup_printf_escaped("<i>%s</i>", list_item);
+				gtk_label_set_markup((GtkLabel*)item_label, italic_text);
+				g_free(italic_text);
+			}
+			g_free(list_item);
+			/* Append item */
+			gtk_menu_shell_append((GtkMenuShell*)statusicon_menu, menu_item);
+			/* Prepare for next item */
+			g_string_free(string, TRUE);
+			if (prefs.reverse_history)
+				element_number--;
+			else
+				element_number++;
+				element_number_small++;
+		}
+		/* Cleanup */
+		g_free(primary_temp);
+		g_free(clipboard_temp);
+		/* Return history to normal if reversed */
+		if (prefs.reverse_history)
+			history = g_slist_reverse(history);
+	}
+	else
+	{
+		/* Nothing in history so adding empty */
+		menu_item = gtk_menu_item_new_with_label(_("Empty"));
+		gtk_widget_set_sensitive(menu_item, FALSE);
+		gtk_menu_shell_append((GtkMenuShell*)statusicon_menu, menu_item);
+	}
+	/* -------------------- */
+	gtk_menu_shell_append((GtkMenuShell*)statusicon_menu, gtk_separator_menu_item_new());
+	/* About */
+	menu_item = gtk_image_menu_item_new_from_stock(GTK_STOCK_ABOUT, NULL);
+	g_signal_connect((GObject*)menu_item, "activate", (GCallback)show_about_dialog, NULL);
+	gtk_menu_shell_append((GtkMenuShell*)statusicon_menu, menu_item);
+	/* Manage history */
+	menu_item = gtk_image_menu_item_new_with_mnemonic(_("_Manage history"));
+	menu_image = gtk_image_new_from_stock(GTK_STOCK_FIND, GTK_ICON_SIZE_MENU);
+	gtk_image_menu_item_set_image((GtkImageMenuItem*)menu_item, menu_image);
+	g_signal_connect((GObject*)menu_item, "activate", (GCallback)show_search, NULL);
+	gtk_menu_shell_append((GtkMenuShell*)statusicon_menu, menu_item);
+	/* Preferences */
+	menu_item = gtk_image_menu_item_new_from_stock(GTK_STOCK_PREFERENCES, NULL);
+	g_signal_connect((GObject*)menu_item, "activate", (GCallback)preferences_selected, NULL);
+	gtk_menu_shell_append((GtkMenuShell*)statusicon_menu, menu_item);
+	/* Quit */
+	menu_item = gtk_image_menu_item_new_from_stock(GTK_STOCK_QUIT, NULL);
+	g_signal_connect((GObject*)menu_item, "activate", (GCallback)quit_selected, NULL);
+	gtk_menu_shell_append((GtkMenuShell*)statusicon_menu, menu_item);
+	/* Popup the menu... */
+	gtk_widget_show_all(statusicon_menu);
+	gtk_widget_set_visible(statusicon_menu, TRUE);
+	gtk_menu_popup((GtkMenu*)statusicon_menu, NULL, NULL, gtk_status_icon_position_menu, status_icon, 1, gtk_get_current_event_time());
+	gtk_menu_shell_select_first((GtkMenuShell*)statusicon_menu, TRUE);
+
+	status_menu_lock = FALSE;
+}
+
+/* Called when status icon is clicked */
+/* (checks type of click and calls correct function) */
 static void status_icon_clicked(GtkStatusIcon *status_icon, GdkEventButton *event )
 {
 	/* Check what type of click was recieved */
@@ -913,14 +820,10 @@ static void status_icon_clicked(GtkStatusIcon *status_icon, GdkEventButton *even
 	/* Left click */
 	else if (event->button == 1)
 	{
-		show_history_menu();
-	}
-	/* Right click */
-	else
-	{
-		show_clipit_menu(status_icon, event->button, gtk_get_current_event_time());
+		show_clipit_menu();
 	}
 }
+
 #endif
 
 /* Called when history global hotkey is pressed */
@@ -938,7 +841,9 @@ void actions_hotkey(char *keystring, gpointer user_data)
 /* Called when actions global hotkey is pressed */
 void menu_hotkey(char *keystring, gpointer user_data)
 {
+#ifndef HAVE_APPINDICATOR
 	show_clipit_menu(status_icon, 0, 0);
+#endif
 }
 
 /* Called when search global hotkey is pressed */
@@ -973,17 +878,17 @@ static void clipit_init()
 	if (!prefs.no_icon)
 	{
 #ifdef HAVE_APPINDICATOR
-	create_app_indicator();
+	create_app_indicator(1);
 #else
 	status_icon = gtk_status_icon_new_from_stock(GTK_STOCK_PASTE);
 	gtk_status_icon_set_tooltip((GtkStatusIcon*)status_icon, _("Clipboard Manager"));
-	g_signal_connect((GObject*)status_icon, "button_release_event", (GCallback)status_icon_clicked, NULL);
+	g_signal_connect((GObject*)status_icon, "button_press_event", (GCallback)status_icon_clicked, NULL);
 #endif
 	}
 }
 
 /* This is Sparta! */
-int main(int argc, char *argv[])
+int main(int argc, char **argv)
 {
 	bindtextdomain(GETTEXT_PACKAGE, CLIPITLOCALEDIR);
 	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
@@ -1049,11 +954,12 @@ int main(int argc, char *argv[])
 	g_free(prefs.actions_key);
 	g_free(prefs.menu_key);
 	g_free(prefs.search_key);
+	g_slist_foreach(history, (GFunc)g_free, NULL);
 	g_slist_free(history);
 	g_free(primary_text);
 	g_free(clipboard_text);
 	g_free(synchronized_text);
-	
+
 	/* Exit */
 	return 0;
 }
