@@ -289,7 +289,7 @@ gboolean show_search()
   GtkWidget* search_dialog = gtk_dialog_new();
 
   gtk_window_set_icon((GtkWindow*)search_dialog, gtk_widget_render_icon(search_dialog, GTK_STOCK_FIND, GTK_ICON_SIZE_MENU, NULL));
-  gtk_window_set_title((GtkWindow*)search_dialog, _("Manage History"));
+  gtk_window_set_title((GtkWindow*)search_dialog, "Manage History");
   gtk_window_set_resizable((GtkWindow*)search_dialog, TRUE);
   gtk_window_set_position((GtkWindow*)search_dialog, GTK_WIN_POS_CENTER);
 
